/***************************************************************************
***************************************************************************/

#include	"always.h"
#include	"../main_src/diva.h"
#include	"complex.h"
#include	"plot_lib.h"
#include	"lam_lib.h"
#include	"vtconfig.h"
#include	"vsyn_lib.h"
#include	"maeda_dir.h"
#include	<fcntl.h>
#include	<sys/ioctl.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<sys/time.h>

#ifdef LINUX_FLAG
 #include <linux/soundcard.h>
 #include <unistd.h>
 #include <fcntl.h>
#else
 #include	<sun/audioio.h>
#endif

/***********************( prototype definitions )**************************/

void	read_rad( void );
void    store_maeda( void );
void    playback_maeda( void );
void    draw_maeda( void );
void	write_sound_buf(char* bufptr, int sampleCount);
void    find_area_function( void );
void	draw_markers( void );
void	open_audio_device( void );
int	select (int, fd_set*, fd_set*, fd_set*, struct timeval *);
void	update_maeda_sliders( void );
void	display_formants( void );

/**********( global variables seen by functions in "maeda.c" )*********/

	int	maeda_init_flag=0;
	int	maeda_count=0;
	int	af_calloc_flag=0;
	FILE*	sound_file_ptr = NULL;
	FILE*	dft_fp = NULL;
	char*	play_back_buffer;
	char*	pbp = NULL;
	float*	fpbp;
	int*	ipbp;
	double	syn_targets[NUM_TARGETS]={220.0, 1220.0, 2220.0/*, 600.0*/};
	extern	int sound_buf_count;
	int	playback_store_count = 0;
	extern	int erase_flag;
	int	synthesizing=0;
	int	record_demo=0;
	int	audio_file_open_flag=0;
	int	audio_fd;
	double	AMpar[7] ={0., 0., 0., 0., 0., 0., 0.};
       #ifndef LINUX_FLAG
	audio_info_t info;
       #endif

/*********************( global plotting variables )************************/

	int	nfrms, nfrmmax = 4;	/* number of formant frequencies  */
	double	frm[8], bw[8], amp[8];	/* formant freqs. and amplitude   */

	vp_lo_frame vp1 = { 1, 0.5, 5.0, 49., 25. }; 	/* area_function  */
	vp_lo_frame vp2 = { 2, 0.5, 32., 49., 37. };    /* VT transfer	  */
	vp_lo_frame vp3 = { 3, 50., 42., 29., 23. };	/* formants	  */
	vp_lo_frame vp4 = { 4, 55., 5.,  35., 35. };	/* VT profile	  */

	char	*UmUg_title = "(Um + Un)/Ug TRANSFER RATIO";
	char	*UmPt_title = "(Um + Un)/Pt TRANSFER RATIO";

/* normalized radiation impedance */
        static  float   *rad_re, *rad_im;
        static  float    dka;
        static  int      n_ka;
        static  double   lip_Rrad, lip_Lrad;
        static  double   nose_Rrad, nose_Lrad;
        static  double   lip_rad_ka, lip_rad_norm;
        static  double   nose_rad_ka, nose_rad_norm;


#define NUM_FRAMES_SAVED 20

/******
*	Function : maeda()
*
*	find_area_function() must be called before calling maeda().  This
*	is usually done in apply_methods().
******/
void    maeda( void )
{
	XEvent	event;
	int	i;

	while (PAUSE == 1)
	{
		while (XtPending() != 0)
		{
			XtNextEvent(&event);
			XtDispatchEvent(&event);
		}
	}

	if (sound_on == 1.0)
		store_maeda();
	else {
		draw_maeda();
		draw_markers();
		XFlush(theDisplay);
		return;
	}

	if (++playback_store_count >= NUM_FRAMES_SAVED) {
		playback_maeda();
	}
}



/******
*	Function : playback_maeda_articulators()
*
*	This is just a faster version of draw_maeda_articulators(), but
*	which doesn't compute and plot formants, doesn't plot the
*	area function, and doesn't redraw the markers, all of which load
*	the system and cause annoying stops in the acoustic output.
******/
void    playback_maeda_articulators( void )
{
	find_area_function();
	deplot_lam();
	plot_lam();

	XFlush(theDisplay);
}



/******
*	Function : draw_maeda_articulators()
******/
void    draw_maeda_articulators( void )
{
	find_area_function();
	get_formants();
	draw_maeda();
	draw_markers();

	XFlush(theDisplay);
}



/******
*	Function : store_maeda()
*
*	This function currently stores one "movement's" worth of sound
*	and formant information in a file for playback.  For performance
* 	reasons, the same information is stored in a RAM playback buffer.
******/
void    store_maeda( void )
{
	int	formants[3];
	int	i;
	float	temp;

	static	int* frame_count_ptr;

	extern	char	sound_buf[8000];

	/* initialize RAM playback buffer */
	if (pbp == NULL) {
		/* initialize pointers for storing in RAM playback buffer */
		frame_count_ptr = (int*)play_back_buffer;
		*frame_count_ptr = 0;
		/* leave space for the frame_count */
		pbp = (char*)(frame_count_ptr + sizeof(*frame_count_ptr));
	}

	if (record_demo) {
		/* store the 3 formants in the file */
		fwrite(syn_targets, sizeof(syn_targets[0]),
						(int)3, sound_file_ptr);

		/* store the 7 maeda parameters in the file */
		fwrite(AMpar, sizeof(AMpar[0]), (int)7, sound_file_ptr);
	}

	/* store the 3 formants in the playback buffer */
	for(i=0; i<3; i++) {
		fpbp = (float*)pbp;
		*fpbp = (float)syn_targets[i];
		pbp += sizeof(float);
	}

	/* store the 7 maeda parameters in the playback buffer */
	for(i=0; i<7; i++) {
		fpbp = (float*)pbp;
		*fpbp = (float)AMpar[i];
		pbp += sizeof(float);
	}

	/* compute speech samples */
	for(i=0; i<NUM_TARGETS; i++)
		formants[i] = (int)syn_targets[i];
	if (sound_on == 1.0)
		/* sensyn() calls writWave() which writes to sound_buf */
		sensyn( formants );

	if (record_demo) {
		/* write next sound buffer to sound file */
		fwrite(&sound_buf_count, sizeof(int), (int)1, sound_file_ptr);
		fwrite(sound_buf,sizeof(char),sound_buf_count,sound_file_ptr);
	}

	/* copy sound buffer to playback buffer */
	ipbp = (int*)pbp;
	*ipbp = sound_buf_count;
printf("sound_buf_count = %d\n",sound_buf_count);
	pbp += sizeof(sound_buf_count);
	memcpy(pbp, (const void *)sound_buf, sound_buf_count);
	pbp += sound_buf_count;

	*frame_count_ptr += 1;
}



/*******************************************************************/
/**	Function : playback_maeda() -- Read from the RAM playback **/
/**     buffer; read maeda parameters and sound samples for each  **/
/**	frame, draw the articulators and play sound using         **/
/**     playback_maeda_articulators().                            **/
/*******************************************************************/

void    playback_maeda( void )
{
	int	num_read;
	char	sound_buf[8000];
	int	sampleCount;
	int*	frame_count_ptr;
	int	i,j;
	float	temp;

	playback_store_count = 0;

	if (sound_on == 0.0) return;

	frame_count_ptr = (int*)play_back_buffer;
	pbp = (char*)(frame_count_ptr + sizeof(*frame_count_ptr));
	for (j=0; j<((int)(*frame_count_ptr)); j++) {
		/* retrieve the formants from the RAM playback buffer */
		for(i=0; i<3; i++) {
			fpbp = (float*)pbp;
			temp = *fpbp;
			syn_targets[i] = (double)temp;
			pbp += sizeof(float);
		}

		/* retrieve maeda parameters from the RAM playback buffer */
		for(i=0; i<7; i++) {
			fpbp = (float*)pbp;
			temp = *fpbp;
			AMpar[i] = (double)temp;
			pbp += sizeof(float);
		}

		if (!synthesizing)
		  playback_maeda_articulators();

		/* retrieve # of sound samples from the RAM playback buffer */
		ipbp = (int*)pbp;
		sampleCount = *ipbp;
		pbp += sizeof(sound_buf_count);

		/* retrieve the sound samples from the RAM playback buffer */
		memcpy((void *)sound_buf, pbp, sound_buf_count);
		pbp += sound_buf_count;

		/* write sound samples to /dev/audio */
		write_sound_buf(sound_buf, sampleCount);
	}

	pbp = NULL;
}



/******************************************************************/
/**	Function : play_demo() --  Read from a stored demo       **/
/**     file the maeda parameters and sound samples for each     **/
/**	frame, draw the articulators, and play the sound.        **/
/******************************************************************/

void    play_demo( FILE *sound_file_ptr )
{
	int	loop_count = 0;
	int	num_read;
	char	sound_buf[8000];
	int	sampleCount;
	int	totalCount;
	int	err;
	int	skipcount=9;
	struct	timeval timeout;

	XtSetSensitive(play_demo_button, FALSE);
	XtSetSensitive(make_demo_button, FALSE);
	XtSetSensitive(speak_button, FALSE);
	XtSetSensitive(babble_button, FALSE);
	XtSetSensitive(formant_button, FALSE);
	XtSetSensitive(ratio_button, FALSE);
	XtSetSensitive(pause_button, FALSE);
	XtSetSensitive(synthesize_button, FALSE);
	XtSetSensitive(cap_button, FALSE);
	XtSetSensitive(fix_jaw_button, FALSE);
	XtSetSensitive(load_weights_button, FALSE);
	XtSetSensitive(load_parameters_button, FALSE);
	XtSetSensitive(plot_conf_button, FALSE);

	open_audio_device();

	totalCount = 0;
	sampleCount = 0;

#ifndef LINUX_FLAG
	info.play.samples = 0;
	ioctl(audio_fd, AUDIO_SETINFO, &info);
#endif
	/*
	 * Read from the file:
	 *
	 * Read maeda parameters and sound samples for each
	 * frame, draw the articulators, and play the sound.
	 */
	while (1) {
		num_read=fread(syn_targets, sizeof(syn_targets[0]), (int)3,
							sound_file_ptr);
		if (num_read == 0) break;
		num_read=fread(AMpar,sizeof(AMpar[0]),(int)7,sound_file_ptr);
		if (num_read == 0) break;
		if (!skipcount--) {
			playback_maeda_articulators();
			skipcount=9;
		}

	/* basically block until enough samples have played */
#ifndef LINUX_FLAG
		ioctl(audio_fd, AUDIO_GETINFO, &info);
#endif
		loop_count = 0; /* but don't block too long */

#ifndef LINUX_FLAG
		while( (totalCount > 400) &&
				(info.play.samples < totalCount-200) ) {
#else
		while( (totalCount > 400) ) {
#endif
			timeout.tv_sec = 0;
			timeout.tv_usec = 5000;

			select( (int)16, (fd_set*)NULL, (fd_set*)NULL,
					(fd_set*)NULL, &timeout);

			if (loop_count++ > 100) break;
#ifndef LINUX_FLAG
			ioctl(audio_fd, AUDIO_GETINFO, &info);
#endif
		}

		num_read=fread(&sampleCount,sizeof(int),(int)1,sound_file_ptr);
		if (num_read == 0) break;
		num_read=fread(sound_buf,sizeof(char),sampleCount,
							sound_file_ptr);
		if (num_read == 0) break;

		/* write next buffer to /dev/audio */
		num_read=write(audio_fd, sound_buf, sampleCount);
		if (num_read == -1) {
			printf("write to audio device failed\n");
			break;
		}
		if (num_read != sampleCount)
			sampleCount = num_read;
		totalCount += sampleCount;
	}

	fclose(sound_file_ptr);
	sound_file_ptr = NULL;

	XtSetSensitive(speak_button, TRUE);
	XtSetSensitive(babble_button, TRUE);
	XtSetSensitive(formant_button, TRUE);
	XtSetSensitive(ratio_button, TRUE);
	XtSetSensitive(synthesize_button, TRUE);
	XtSetSensitive(pause_button, TRUE);
	XtSetSensitive(cap_button, TRUE);
	XtSetSensitive(fix_jaw_button, TRUE);
	XtSetSensitive(load_weights_button, TRUE);
	XtSetSensitive(load_parameters_button, TRUE);
	XtSetSensitive(plot_conf_button, TRUE);
	XtSetSensitive(play_demo_button, TRUE);
	XtSetSensitive(make_demo_button, TRUE);

	shift_eye();
	flap_arms();

	update_maeda_sliders();
	display_formants();
}



/*********************************************************************/
/**	Function : find_area_function() -- set up some stuff, then  **/
/**     call lam() to get vt profile and sagittal_to_area() and     **/
/**	appro_area_function() to get area function (these functions **/
/**     are in lam_lib.c).                                          **/
/*********************************************************************/

void    find_area_function( void )
{
		int	i,j, p=0;
		int	ns0 = 29;
	static	area_function	*af0;

	if (af_calloc_flag==0) {
		af0 = (area_function *) calloc( ns0, sizeof(area_function) );
		nph = 9;
		nbu = 8;
		nss = nbu + nph;
		afvt  = (area_function *) calloc( nss, sizeof(area_function) );
		af_calloc_flag = 1;
	}

	/* Compute VT profile and area function */
	lam( AMpar );				/* profile	*/
	sagittal_to_area( &ns0, af0 );		/* area function */

	appro_area_function( ns0, af0, nss, afvt);

	if( nasal_tract == ON )
	{  anc = (double) min( anc, afvt[nph].A );
	   afvt[nph].A -= anc;
	}
}


/*********************************************************************/
/**	Function : draw_maeda() -- erases (usually) and draws maeda **/
/**     articulators and area function (using plot_lam() and        **/
/**     plot_af(),respectively, from lam_lib.c).                    **/
/**     find_area_function() must be called before draw_maeda().    **/
/*********************************************************************/

void    draw_maeda( void )
{
	/* draw maeda articulators */
	if (erase_flag) deplot_lam();
	plot_lam();

	/* plot area function */
	deplot_af();
	plot_af ( nss, afvt );
}



void init_maeda( void )
{
	static  char    *nt_filename = "nt.are";
	static  char	af_path[40];
	int	j;

	maeda_count = 0;

	if (maeda_init_flag == 0) {
		/* read nasal tract area function from the file */
		strcpy(af_path, VTAFdir);
		strcat(af_path, nt_filename);
		read_af( af_path, &nna, &afnt );
		read_rad();

		/* Initialization */
	 	read_short_model_spec();
		/* read_model_spec();  Replaced by FG; 7/10/96 */
		convert_scale();
		semi_polar();
		/* init_rbf(); */
		for (j=0; j<7; j++) AMpar[j] = 0.0;

		maeda_init_flag = 1;
	}

	if ((play_back_buffer = (char*)malloc(
	    NUM_FRAMES_SAVED*(400 + (3+7)*sizeof(double)))) == NULL) {
		printf("unable to malloc playback buffer\n");
		exit(0);
	}
}



void write_sound_buf(char* bufptr, int sampleCount)
{
    int err;

    if (sound_on == 0.0) return;
    open_audio_device();
    write(audio_fd, bufptr, sampleCount);
}



void	open_audio_device( void )
{
  int div = 4;

	if (audio_file_open_flag == 0) {
		if ((audio_fd=open("/dev/audio", O_RDWR)) == -1) {
			printf("can't open /dev/audio\n");
			exit(1);
		}
		audio_file_open_flag = 1;

#ifdef LINUX_FLAG
		  /* Shorten the DMA buffer to 1024 bytes for linux */
		  ioctl(audio_fd, SOUND_PCM_SUBDIVIDE, &div);
#else
		  AUDIO_INITINFO(&info);
#endif
	}
}
