#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <X11/Xlib.h>
#include <Xm/Xm.h>

/******** Parameters (see also parameters.dat) ***********/
extern double which_fmap;
extern double phoneme_learning_rate;
extern double phoneme_decay_rate;
extern double velocity_learning_rate;
extern double velocity_decay_rate;
extern double trial_length;
extern double time_step;
extern double lip_perturb_height;
extern double jaw_perturb_height;
extern double jaw_fixed_height;
extern double num_velocity_trials;
extern double num_phoneme_trials;
extern double min_cube_factor;
extern double coartic_distance;
extern double scale;
extern double stop_after_phoneme;
extern double babbling_graphics_on;
extern double sound_on;
/*********** End parameters ***********************/

#define RBF 1

#define INTEGRATOR rk4
#define NUM_METHODS 14

/*  state variables - each symbol defines an agonist/antagonist pair */
#define FMF1	1	/* formant F1 */
#define FMF2	FMF1+2	/* formant F2 */

/* these state variables are not currently being used */
#define FMF3	FMF2+2	/* formant F3 */
#define TDCL	FMF3+2	/* tongue dorsum constriction location */
#define TDCD	TDCL+2	/* tongue dorsum constriction degree */
#define TTCL	TDCD+2	/* tongue tip constriction location */
#define TTCD	TTCL+2	/* tongue tip constriction degree */
#define LPRO	TTCD+2	/* lip protrusion */
#define LAPR	LPRO+2	/* lip aperture */
#define TBPR	LAPR+2	/* tongue body pressure receptors */
#define TTPR	TBPR+2	/* tongue tip pressure receptors */
#define LLPR	TTPR+2	/* lower lip pressure receptors */
#define ULPR	LLPR+2	/* upper lip pressure receptors */
#define VELH	ULPR+2	/* velum height */

#define NUM_STATE_VARS 6	/* VELH+1 */

#define NUM_PHONEMES 11
#define NUM_ARTICULATORS 7

#define NUM_TARGETS 3

#define MAX_ATTEMPT_LENGTH 105

extern double F0,AV,AF;
extern int USE_RATIO_COORDS;
extern int ANIMATION_SPEED;
extern int SPEAKING_RATE;
extern int BREAK;
extern int CONTINUE_AFTER_PHONEME;
extern int PAUSE;
extern char PHONEME_STRING[];
extern char *CURRENT_PARAMETER;
extern char BABBLE_TYPE[];
extern int BLOCKED[];

extern double FORMANT_TARGET[];
extern double RATIO_TARGET[];
extern double IDEAL_FORMANTS[NUM_PHONEMES+1][NUM_TARGETS];
extern double IDEAL_RATIOS[NUM_PHONEMES+1][NUM_TARGETS];
extern double MAX_RATIO[NUM_PHONEMES+1][NUM_TARGETS];
extern double MIN_RATIO[NUM_PHONEMES+1][NUM_TARGETS];
extern double syn_targets[NUM_TARGETS];
extern double s[NUM_STATE_VARS + 1];



extern int MARKER_COUNT;
extern int MARKER[50][2];

extern int param_count;

extern unsigned long theWhitePixel;
extern unsigned long theBlackPixel;
extern GC theGC;
extern Window theWindow;
extern Display *theDisplay;
extern Pixel bg_color;

extern Widget top_level, form, drawing_area, control_panel, translation_panel,
  speak_button, formant_button, ratio_button, babble_button, coord_frame_button,
  break_button, reset_config_button, param_button, fix_f0_button,
  maeda_panel, param_value_string, pause_button, fix_jaw_button, cap_button,
  load_weights_button, save_weights_button, load_parameters_button,
  save_parameters_button, change_parameters_button, quit_button,
  synthesize_button, plot_conf_button, play_demo_button,
  make_demo_button, print_config_button;

extern double lower_lip_height;
extern double upper_lip_height;
extern double tongue_body_shape;
extern double tongue_tip_height;

extern double OLD_ARTIC_POS[NUM_ARTICULATORS + 1][2]; 
extern double ARTIC_POS[NUM_ARTICULATORS + 1][2];   /* 1 = jaw
						       2 = tongue body
						       3 = tongue tip
						       4 = bottom lip
						       5 = top lip
						       6 = velum */

extern double velum_height;
extern double x_velum_height;
extern double jaw_height;
extern double lip_protrusion;
extern double larynx_height;
extern double lip_aperture;
extern double tongue_tip_position;
extern double tongue_body_position;

extern double x_lower_lip_height;
extern double x_upper_lip_height;
extern double x_lower_lip_location;
extern double x_upper_lip_location;
extern double x_tongue_body_height;
extern double x_tongue_tip_height;
extern double x_tongue_tip_y_location;
extern double x_tongue_body_y_location;

extern double AMpar[7];
		/* 0 ==> jaw height */
		/* 1 ==> tongue body position */
		/* 2 ==> tongue body shape */
		/* 3 ==> tongue tip position */
		/* 4 ==> lip aperture */
		/* 5 ==> lip protrusion */
		/* 6 ==> larynx height */

extern char perturb_lip;
extern char perturb_jaw;
extern int tip_dental_stop;
extern int LIP_PERT;
extern int JAW_PERT;
extern int FIX_JAW;
extern int FIX_F0;
extern int FORMANT_TARGET_FLAG, RATIO_TARGET_FLAG;                    
extern int velum;
extern int voicing;
extern int noise;
extern int pressure;

extern char p_name[];
extern int p_count[];
extern int p[];
extern double m[], x[];
