#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "rbf.h"
#include "../main_src/diva.h"
#include "../main_src/babble.h"
/* #include <console.h> */
 
#define Errorlim   0.005	/*.003*/
#define jk numin*j+k 

extern  void    srand48( long seedval ) ;
extern  double  drand48( ) ;
extern  int     get_formants() ;

double STEPSIZE;    

/********  These are declared for external use in rbf.h  ********/

rbf_struct *CreateRbf();
void    TrainRbf( rbf_struct );
void    RbfInvMap(double *,double *,double *, rbf_struct *);
void    RbfForMap(double *,double *, rbf_struct *);
void	SaveWeights(FILE *, rbf_struct * ) ;
void	ReadWeights( FILE *,rbf_struct * ) ;
void    FreeRbf( rbf_struct * );
void    NewInvMap(rbf_struct * );
void    NormVec ( double *, int ) ;
void    PrintBias( rbf_struct *, double *);
rbf_struct *P;

void	OldSaveWeights(FILE *, rbf_struct * ) ;
void	OldReadWeights( FILE *,rbf_struct * ) ;

/*********  These are only visible to this file   ***************/

void    Fmap( rbf_struct *P ) ;
void    Pmap( rbf_struct *P ) ;
void    ApplyJInv( rbf_struct *P) ;
void    ApplyForMap(rbf_struct *P) ;
int     GetNearPos( rbf_struct *, double *) ;
int     GetStep( rbf_struct *) ;
void    InitMaps( rbf_struct *P ) ;



/*****************  CreateRbf()  ****************************************/
/*   Allocates the space for an rbf structure, initializes it, and 
      returns a pointer to it. 

   WARNING:  In the call to InitMaps(), this function messes with 
    the global variable AMpar[]


*/

rbf_struct *CreateRbf()
{
  rbf_struct *P;int a;
  
  P = (rbf_struct *)malloc(sizeof(rbf_struct));
  InitMaps(P);
  printf("numJin: %d\n",P->numJin);
  return(P);
}





/****************  TrainRbf() ***********************************/
/*   Passed an rbf structure (which has been initialized), this 
     routine trains both the forward and inverse maps using 
     the dictionary to estimate error.  

   Precond:  Maeda must be initialized and the dict loaded    

   WARNING: This routine messes with the global variables AMpar[] and 
            syn_targets[].


   Note:the difference between a step & a reach is that error info is
    printed after each reach, and during learning of the forward
    map, movement direction changes with each reach.  Error sums 
    are cleared between epochs.


*/

void TrainRbf( rbf_struct P)
{
       
 long   seedval ;
 int    numin, i, j, k, step, reach, a,  gooddata ;
 double  *in, *target, *out,testin[7], testtarget[NUM_TARGETS] ;
 double  *bad, *Error_list, StartTarget[NUM_TARGETS] ;
 double  errtot[NUM_TARGETS], errorsum[NUM_TARGETS], Jerrtot[NUM_TARGETS], Jerrorsum[NUM_TARGETS], targsum[NUM_TARGETS], Jtargsum[NUM_TARGETS], Error ;
 time_t tloc ;
 FILE *wtfile, *NewBasis ;
 double save_pos[7];

 seedval = (long) time(&tloc) ;
 srand48( seedval );
 
/*********  Train Forward Map ***************************************/

 if(NUM_TARGETS != NUM_STATE_VARS/2){
   printf("Error!!  NUM_TARGETS and NUM_STATE_VARS incompatible - can't train\n");
   return;
 }

 which_fmap = 2.0; /* set to Maeda */


 for(k=0;k<P.numin;k++)
 {
   AMpar[k] = 0.0 ;
   P.posn[k] = 0.0 ;
 }


 /*
 AMpar[0] = 0.0;AMpar[1] = 1.5;AMpar[2] = -0.25; AMpar[3] = -0.1; 
 AMpar[4] = -1.2;AMpar[5] = 0.1; AMpar[6] = -0.4;
 for(k=0;k<P.numin;k++)
 {
   P.posn[k] = AMpar[k] ;
 }
 */

 get_formants();
 for(k=0;k<P.numtarget;k++)
 {
   StartTarget[k] = syn_targets[k] ;
   P.target[k] = syn_targets[k] ;
 } 

 Error = 1.0;

 P.epochs = 0 ;

 while( (P.epochs < (P.numEpochs/2.0)) && (Error > Errorlim) && (BREAK != 1) )
 {

  for(i=0 ; i<NUM_TARGETS ; i++) {
   errorsum[i] = 0. ;
   targsum[i] = 0. ;
  }

  for(reach = 0; (reach<P.numreach) && (Error > Errorlim) &&(BREAK!=1); reach++)
  {
   for(k=0;k<P.numin; k++)
     P.direct[k] = P.range[k]*(drand48()-0.5)/20.0 ;  

   for(step = 0; step< P.numstep; step++)
   {
    if(look_for_break()) break;

    gooddata = 0;
    while (!gooddata)  gooddata = GetNearPos( &P,StartTarget ) ;
/*    for(k=0;k<7;k++)printf("%lf ",P.posn[k]);printf("\n");*/
    Fmap( &P ) ;   /* learns SCALED forward map */
/*    printf("%lf %lf %lf\n",P.M[0].error,P.M[1].error,P.M[2].error);    */


    for(a = 0; a<P.numFMaps; a++)
    { 
      errorsum[a] += fabs(P.M[a].error) ;  /* both error & targ  are */
      targsum[a] += fabs(P.M[a].target) ;  /*  in scaled coords      */
    }

   }  /* end for ( step */
  }  /* end for ( reach */

  Error = 0;

  for(i = 0; i < P.numJin; i++) {
   errtot[i] = errorsum[i]/targsum[i] ;
   printf( "F %6.2lf  (%6.2lf, %6.2lf)", errtot[i],errorsum[i],targsum[i] ) ;
   Error += (errtot[i]/P.numJin);
  }
  printf("\n\n") ;
  

  for(i=0 ; i<P.numJin ; i++)
   printf("%6.1lf  ", P.M[i].out /*P.scale[i]*/ ) ;  /* unscale fmap output*/
  printf("\n") ;
  for(i=0 ; i<P.numJin ; i++)
   printf("%6.1lf  ", P.prevtarget[i]) ;        /*compare to frmt target*/
  printf("\n\n") ;


  P.epochs ++ ;
 } /* end while */

 if(Error <= Errorlim) 
    printf("Error low enough\n");  
 else
    printf("done num Epochs \n");

 which_fmap = 1.0;  /* set so that rbfs are used for the forward map */
                    /* 0: dict, 1:rbfs, 2:maeda*/

/***************Train Inverse Map ***********************************/
 BREAK = 0;
 /*NewInvMap(&P);*/
 Error = 1.0;

 P.epochs = 0 ;
 while( (P.epochs < P.numEpochs) && (Error > Errorlim) && (BREAK != 1) )
 {

  for(i=0 ; i<NUM_TARGETS ; i++) {
   Jtargsum[i] = 0. ;
   Jerrorsum[i] = 0. ;
  }
  
  for(reach = 0; (reach<P.numreach) && (Error > Errorlim) &&(BREAK!=1); reach++)
  {

   for(step = 0; step<P.numstep; step++)
   {
    if(look_for_break()) break;
    gooddata = 0;
    while (!gooddata)  gooddata = GetStep( &P ) ;

#ifdef COMMENT
 
    printf("?\n");
    getchar();

    printf("step artic:");
    for(i=0;i<7;i++)
      printf("%lf ",P.Jtarget[i]);
    printf("\nfrom: ");
    for(i=0;i<7;i++)
      printf("%lf ",P.previn[i]);
    printf("\nto  : ");
    for(i=0;i<7;i++)
      printf("%lf ",P.in[i]);
    printf("\n");

    printf("step in state space:");
    for(i=0;i<3;i++)
      printf("%lf ",P.Jin[i]);


    printf("\nto  : ");
    for(i=0;i<3;i++)
      printf("%lf ",P.target[i]);
    printf("\n");

    printf("est'd step artic:"); 
    for(i=0;i<7;i++)
      printf("%lf ",P.Jout[i]);
    printf("\n");
#endif COMMENT


 
   Pmap( &P ) ;   /* learns UNSCALED inv map  */
    
   for ( k=0; k<P.numin; k++ ){
      AMpar[k] = P.previn[k] + /*P.stepsize*/P.Jout[k] ;
      /*      printf("%lf ",P.Jout[k]);*/
   }
   /*    printf("\n");*/

    get_formants() ;
    get_state();

    for(i = 0; i < P.numJin; i++)
        P.Jerror[i] = (s[2*i+1]-s[2*i+2]) ;   /* the prediction in state space*/

    for(i=0;i<P.numJin;i++){
      syn_targets[i] = P.target[i];    
    }
    get_state();

    for(i = 0; i < P.numJin; i++)            /*          minus                */
        P.Jerror[i] -= (s[2*i+1]-s[2*i+2]) ; /* the actual posn in state space*/

    
    for(i = 0; i < P.numJin; i++)
    {
       Jerrorsum[i] += fabs(P.Jerror[i]) ;
       Jtargsum[i] += fabs(P.Jin[i]) ;
    }
 

   }  /* end for ( step */
  }  /* end for ( reach */

  for(i = 0; i < P.numJin; i++) {
    if(Jtargsum[i]!= 0) Jerrtot[i] = Jerrorsum[i]/Jtargsum[i] ;
    else Jerrtot[i] = Jerrorsum[i];
    Error += (Jerrtot[i]/P.numJin);
    printf( "I %lf (%lf %lf)", Jerrtot[i], Jerrorsum[i],Jtargsum[i] ) ;
  }
  printf("\n\n") ;

  for(i=0 ; i<P.numJin ; i++)
   printf("%5.2lf  ", (P.Jin[i] + P.Jerror[i]) ) ; 
  printf("\n") ;
  for(i=0 ; i<P.numJin ; i++)
   printf("%5.2lf  ", P.Jin[i] ) ;
  printf("\n\n") ;
 
  Error = Error/2.0;


  P.epochs ++ ;
 } /* end while */

 if(Error <= Errorlim) 
    printf("Error low enough\n");  
 else
    printf("done num Epochs \n");


/**  Done basic training, now check if inverse map needs more bases, 
     if so add them and train around centers  (code in add1.c)       */
  BREAK = 0;

#ifdef COMMENT
  NewBasis = fopen("NewBasis","w");

  bad = (double *)calloc(P.numin*3,sizeof(double)) ;
  Error_list =(double *)calloc(3,sizeof(double));
  if(!ComputeError(P,bad,Error_list,3)) printf("Bad return value \n");
  while(Error_list[0] > 90){
      if (BREAK) break ;
      for(j=0;j<3;j++){
         printf(" Error is %lf at ",Error_list[j]);
         for(i=0;i<P.numin;i++){
           printf(" %.4lf",bad[j*P.numin + i]);
	}
         printf("\n");
      }
       printf("\n");

      for(j=0;j<3;j++)
         if(!PutBasis(P,bad + j*P.numin)) printf("AAAGH\n");

      if(Error_list[2] < 45) break ;
      if(!ComputeError(P,bad,Error_list,3)) printf("Bad return value \n");
   }
  fclose(NewBasis);
#endif COMMENT

 
 if(wtfile = fopen("rbf_weights.new","w"))
   {
    printf("Saving rbf wts to rbf_weights.new\n");
    SaveWeights( wtfile,&P );
    fclose(wtfile);
 }
 else printf("Problem opening rbf_weights.new - WTS NOT SAVED\n") ;
 
}  /* end main */



/********************   InitMaps    *****************************
  
  The way rbf structures are set up, each entry in the inverse Jacobian 
  could have a different number of basis functions, but this is not used 
  in the current implementation.  The forward and inverse maps can be started 
  with different numbers of rbfs however, and PutBasis() only adds rbfs to 
  the inverse map.  

PreCond: The parameter P must pt to an allocated rbf structure.


WARNING:   This function messes with the global var AMpar[]

*/

void InitMaps( rbf_struct *P )
{
 int a, i, j, k, *numseg ;
 int  index[7],  maxout, numin, MAXHYP, numhyp ;
       
  P->numFMaps = NUM_TARGETS ;
  P->numPMaps = NUM_TARGETS*7 ;   /* number of weights in inv. Jacobian */
  P->numBMaps = 7 ; 
  P->numstep = 10 ;
  P->numreach = 12;
  P->numEpochs = 55 ;
  P->MAXHYP =  150;
  P->etaF = .15 ;
  P->metaF = 0.0000;  /*.004 ;*/    /*  NOT SUPPORTED  */
  P->etaP_max = 5.0 ;/* .00005 ;*/
  P->etaP = P->etaP_max;  
  P->metaP = 0.000;  /*.0000005;*/   /* NOT SUPPORTED  */
  P->etaB = 0.05 ;  /*0.05;*/
  P->metaB = 0.0000 ;
  /*
  P->scale[0] = 1.0 ;
  P->scale[1] = 1.0 ;
  P->scale[2] = 1.0 ;
  */
  P->numin = 7 ;     /* number of theta */
  P->numtarget = NUM_TARGETS ; /* num of x */
  P->numJin = NUM_TARGETS ;    /* number of x-dots */ 
  P->numJout = 7 ;   /* number of theta-dots */
  P->maxdiff = 25.0 ;
  
  P->rangemin = (double *)calloc(P->numin,sizeof(double));
  P->rangemax = (double *)calloc(P->numin,sizeof(double));
  P->range = (double *)calloc(P->numin,sizeof(double));

  /* now set ranges based on the constants in babble.h */

  P->rangemin[0] = MIN_JAW_HEIGHT;
  P->rangemax[0] = MAX_JAW_HEIGHT;
  P->rangemin[1] = MIN_TONGUE_BODY_POSITION;
  P->rangemax[1] = MAX_TONGUE_BODY_POSITION;
  P->rangemin[2] = MIN_TONGUE_BODY_SHAPE;
  P->rangemax[2] = MAX_TONGUE_BODY_SHAPE;
  P->rangemin[3] = MIN_TONGUE_TIP_POSITION;
  P->rangemax[3] = MAX_TONGUE_TIP_POSITION;
  P->rangemin[4] = MIN_LIP_APERTURE;
  P->rangemax[4] = MAX_LIP_APERTURE;
  P->rangemin[5] = MIN_LIP_PROTRUSION;
  P->rangemax[5] = MAX_LIP_PROTRUSION;
  P->rangemin[6] = MIN_LARYNX_HEIGHT;
  P->rangemax[6] = MAX_LARYNX_HEIGHT;

  for(k=0;k<P->numin;k++)
     P->range[k] = P->rangemax[k] - P->rangemin[k] ;

  P->M = (FMap *)calloc(P->numFMaps, sizeof(FMap) ) ;
  P->J = (PMap *)calloc(P->numPMaps, sizeof(PMap) ) ;
  P->B = (BMap *)calloc(P->numBMaps, sizeof(BMap) ) ;

  for(a=0 ; a<P->numFMaps ; a++ ) {
   P->M[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ; }
  for(a=0 ; a<P->numPMaps ; a++ ) {
   P->J[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ; }
  for(a=0 ; a<P->numBMaps ; a++ ) {
   P->B[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ; }
  
  for(a=0; a<P->numFMaps; a++)
  {
	  P->M[a].numseg[0] = 2 ;
	  P->M[a].numseg[1] = 3 ;
	  P->M[a].numseg[2] = 2 ;
	  P->M[a].numseg[3] = 2 ;
	  P->M[a].numseg[4] = 3 ;
	  P->M[a].numseg[5] = 1 ;
	  P->M[a].numseg[6] = 1 ;
  }
  for(a=0; a<P->numPMaps; a++)
  {
	  P->J[a].numseg[0] = 2 ;
	  P->J[a].numseg[1] = 3 ;
	  P->J[a].numseg[2] = 2 ;
	  P->J[a].numseg[3] = 2 ;
	  P->J[a].numseg[4] = 3 ;
	  P->J[a].numseg[5] = 1 ;
	  P->J[a].numseg[6] = 1 ;
  }
 for(a=0; a<P->numBMaps; a++)
  {
	  P->B[a].numseg[0] = 2 ;
	  P->B[a].numseg[1] = 3 ;
	  P->B[a].numseg[2] = 2 ;
	  P->B[a].numseg[3] = 2 ;
	  P->B[a].numseg[4] = 3 ;
	  P->B[a].numseg[5] = 1 ;
	  P->B[a].numseg[6] = 1 ;
  }


  /* alpha determines how much the B vector learns an inward bias  */
  for(a = 0; a<P->numBMaps ; a++){
    P->B[a].alpha = 0.03 ; /*.03*/ ;
 }
  /*
  P->B[1].alpha = .006;
  P->B[3].alpha = .006;
  */
  MAXHYP = P->MAXHYP ;
  numin = P->numin ;
  maxout = P->numJout ;
  P->numsamp = 0. ;

  P->stepsize = .1*P->range[1]; /* used to divide by P->numstep*/ 
  STEPSIZE = P->stepsize ;

  
  P->c = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
  P->linfit = (double *)calloc(MAXHYP, sizeof(double) ) ;
  P->direct = (double *)calloc(numin, sizeof(double) ) ;
  P->in = (double *)calloc(numin, sizeof(double) ) ;
  P->previn = (double *)calloc(numin, sizeof(double) ) ;
  P->posn = (double *)calloc(numin, sizeof(double) ) ;
  P->target = (double *)calloc(NUM_TARGETS, sizeof(double) ) ;
  P->prevtarget = (double *)calloc(NUM_TARGETS, sizeof(double) ) ;
  P->Jin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->avgJin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->Jout = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jtarget = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jerror = (double *)calloc(P->numJout, sizeof(double) ) ;
  
  for(a=0; a<P->numFMaps; a++)
  {
   numin = P->numin ;
   P->M[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->M[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
   P->M[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
   P->M[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->M[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
}

	 
  for(a=0; a<P->numPMaps; a++)
  {
   numin = P->numin ;
   P->J[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->J[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
   P->J[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
   P->J[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->J[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
  }
  
  for(a=0; a<P->numBMaps; a++)
  {
   numin = P->numin ;
   P->B[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->B[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
   P->B[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
   P->B[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
   P->B[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
  }  


 for(a=0; a<P->numFMaps; a++)
 {
  numseg = P->M[a].numseg ;
  numin = P->numin ;
	 j = 0 ;
	 
	 for ( index[0]=0; index[0]<numseg[0]; index[0]++ )
	 {
	  for ( index[1]=0; index[1]<numseg[1]; index[1]++ )
	  {
	   for ( index[2]=0; index[2]<numseg[2]; index[2]++ )
	   {
	    for ( index[3]=0; index[3]<numseg[3]; index[3]++ )
	    {
	     for ( index[4]=0; index[4]<numseg[4]; index[4]++ )
	     {
	      for ( index[5]=0; index[5]<numseg[5]; index[5]++ )
	      {
	       for ( index[6]=0; index[6]<numseg[6]; index[6]++ )
	       { 
	        for(k=0 ; k<numin ; k++)
	        {
	         if(numseg[k]<=1)
	         {
	          P->M[a].mu[jk] =P->rangemin[k] + P->range[k]/2.0 ;
	          P->M[a].sig[jk] = P->range[k]/2.0 ;
	         }
	         else
	         {
	          P->M[a].mu[jk] = P->rangemin[k] + index[k]*P->range[k]/(numseg[k]-1) ;
	          P->M[a].sig[jk] = (P->range[k]/(numseg[k]-1))/2.0 ;
	         }
                 AMpar[k] = P->M[a].mu[jk] ;
	        }
	        /*if(check_valid_configuration()) */ j++ ;
	       }
	      }
	     }
	    } 
	   }
	  }
	 }
 if(j<= MAXHYP) P->M[a].numhyp = j;
 else {
      printf("Error : exceeding max number of bases\n");
      P->M[a].numhyp = MAXHYP ;
   }
}

 for(a=0; a<P->numPMaps; a++)
 {
  numseg = P->J[a].numseg ;
  numin = P->numin ;
	 j = 0 ;
	 
	 for ( index[0]=0; index[0]<numseg[0]; index[0]++ )
	 {
	  for ( index[1]=0; index[1]<numseg[1]; index[1]++ )
	  {
	   for ( index[2]=0; index[2]<numseg[2]; index[2]++ )
	   {
	    for ( index[3]=0; index[3]<numseg[3]; index[3]++ )
	    {
	     for ( index[4]=0; index[4]<numseg[4]; index[4]++ )
	     {
	      for ( index[5]=0; index[5]<numseg[5]; index[5]++ )
	      {
	       for ( index[6]=0; index[6]<numseg[6]; index[6]++ )
	       {
	        for(k=0 ; k<numin ; k++)
	        {
	         if(numseg[k]<=1)
	         {

	          P->J[a].mu[jk] = P->rangemin[k] + P->range[k]/2.0 ;
	          P->J[a].sig[jk] = P->range[k]/2.0 ;
/*
	          P->J[a].mu[jk] = -2.9 + 4.8/2.0 ;
	          P->J[a].sig[jk] = 4.8/2.0 ;
*/
	         }
	         else
	         {

	          P->J[a].mu[jk] = P->rangemin[k] + index[k]*P->range[k]/(numseg[k]-1) ;
	          P->J[a].sig[jk] = (P->range[k]/(numseg[k]-1))/2.0 ;
/*
	          P->J[a].mu[jk] = -2.9 + index[k]*4.8/(numseg[k]-1) ;
	          P->J[a].sig[jk] = (4.8/(numseg[k]-1))/2.0 ;	      
*/  
                 }	
                 /*  AMpar[k] = P->J[a].mu[jk] ;*/
                }
               /* if(check_valid_configuration()) */j++;
	       }
	      }
	     }
	    } 
	   }
	  }
       }
 if(j<= MAXHYP) P->J[a].numhyp = j;
 else {
      printf("Error : exceeding max number of bases\n");
      P->J[a].numhyp = MAXHYP ;
   }
}



 for(a=0; a<P->numBMaps; a++)
 {
  numseg = P->B[a].numseg ;
  numin = P->numin ;
	 j = 0 ;
	 
	 for ( index[0]=0; index[0]<numseg[0]; index[0]++ )
	 {
	  for ( index[1]=0; index[1]<numseg[1]; index[1]++ )
	  {
	   for ( index[2]=0; index[2]<numseg[2]; index[2]++ )
	   {
	    for ( index[3]=0; index[3]<numseg[3]; index[3]++ )
	    {
	     for ( index[4]=0; index[4]<numseg[4]; index[4]++ )
	     {
	      for ( index[5]=0; index[5]<numseg[5]; index[5]++ )
	      {
	       for ( index[6]=0; index[6]<numseg[6]; index[6]++ )
	       {
	        for(k=0 ; k<numin ; k++)
	        {
	         if(numseg[k]<=1)
	         {
	          P->B[a].mu[jk] = P->rangemin[k] + P->range[k]/2.0 ;
	          P->B[a].sig[jk] = P->range[k]/2.0 ;
	         }
	         else
	         {
	          P->B[a].mu[jk] = P->rangemin[k] + index[k]*P->range[k]/(numseg[k]-1) ;
	          P->B[a].sig[jk] = (P->range[k]/(numseg[k]-1))/2.0 ;
	         }
                /* AMpar[k] = P->B[a].mu[jk] ;*/
	        }
	       /*if(check_valid_configuration())*/ j++ ;
	       }
	      }
	     }
	    } 
	   }
	  }
       }
  if(j<= MAXHYP) P->B[a].numhyp = j;
  else {
      printf("Error : exceeding max number of bases\n");
      P->B[a].numhyp = MAXHYP ;
   }
}


}


/********************  Fmap  *************************************

The routine which processes one training pt for the forward map.   
The forward map learns to map an artic posn to a scaled formant posn (NOT 
translated to state space). The scaling allows us to emphasize accuracy 
for lower formants.



Precond:  P must be initialized, the P->posn array must hold a posn in artic 
          space and each P->M[i].target must hold the corresponding scaled 
          posn (divided by P->scale[i]) along the ith acoustic dimension.

Postcond: P->M[i].out will hold the rbfs' current estimate of scaled posn along 
          acoustic dimension i, P->M[i].error will indicate how far off 
          the target this is, and a gradient descent learning step will have 
          been applied to P->M[i].v, & P->M[i].w[] accordingly.
          P->M[i].mu and sig are also updated, but the learning rate parameters
          for them are currently set to zero.



Warning:  Current version assumes the centers and sigs of the rbfs are in 
          the same places for all M entries - if adaptive rbfs are 
          used, must rewrite this code.
  
**********************************************************************/



void Fmap( rbf_struct *P )
{
 int  i, j, k, a, b, n ;
 double diffmuin, hsum, *h ;
 double eta, meta, dmu, out, target, error ;
 double *c, *w, *v, *sig, *mu, *in, linfit ;
 int   numhyp, numin ;

  numin = P->numin ;
  numhyp = P->M[0].numhyp;
  c = (double *)calloc(numin*numhyp,sizeof(double));
  h = (double *)calloc(numhyp,sizeof(double));
  in = P->posn ;
  eta = P->etaF ;
  meta = P->metaF ;
   

   mu = P->M[0].mu ;
   sig = P->M[0].sig ;
   hsum = 0. ;
   for ( j=0; j<numhyp; j++ )
   {
     diffmuin = 0. ;  
     for ( k=0; k<numin; k++ )
     {
      n = jk ;
      c[n] = (in[k] - mu[n])/sig[n] ;
      diffmuin += c[n]*c[n] ;
     }
     
     if( diffmuin < P->maxdiff ) 
      h[j] = exp( -diffmuin ) ;
     else
      h[j] = 0.0 ; 
     hsum += h[j] ;     
   }     

   if(hsum > 0.0)
   {
    for ( j=0; j<numhyp; j++ ) {
     h[j] /= hsum ; }
   }
   else
    printf("Error\n") ;

  for(a = 0; a<P->numFMaps; a++)
  {
   w = P->M[a].w ;
   v = P->M[a].v ;
   target = P->M[a].target ;
   

   out = 0. ;
  
 
    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk] ;
      }
      out += h[j]*(v[j] + linfit) ;
    }
     
    error = target - out ;
 
 /*      printf("error = %lf\n", *error) ; */
    for( j=0; j<numhyp; j++ )
    {
     if( h[j]>0.0 )
     {
      v[j] += eta*error*h[j] ;
      for( k=0; k<numin; k++ )
      {
       n = jk ;
       w[n] += eta*error*c[n]*h[j] ;
      }
     }
    }
   P->M[a].out = out ;  
   P->M[a].error = error ;

   }  /* end for a */
  free(c);
  free(h);
}



/********************  Pmap  *************************************
The routine which processes one training pt for the inverse map.   
The inverse map learns a \delta state space to \delta artic map.


Precond: 1. P must be initialized
         2. P->previn must contain starting posn in artic space (necess. 
                    because J-1 depends on artic posn)
         3. P->Jin contains the \delta state space
         4. P->Jtarget contains the corresponding \delta artic

Postcond: P->Jout will hold the rbfs' current estimate of the \delta artic,
          the P->Jerror vector will indicate how far off the target 
          this is,and a gradient descent learning step will have been applied 

          to all the P->J[i].v, P->J[i].w[],P->B[i].v, &P->B[i].w[] accordingly.
          The mu and sig are also updated, but the learning rate parameters
          for them are currently set to zero.

Warning:  Current version assumes the centers and sigs of the rbfs are in 
          the same places for all J and B entries - if adaptive rbfs are 
          used, must rewrite this code.
  
**********************************************************************/


void Pmap( rbf_struct *P )
{
int  i, j, k, a, b, e, n ;
double diffmuin, hsum, *h ;
double eta, meta, dmu, kout[NUM_TARGETS][7] ;
double *c, *w, *v, *sig, *mu, out, target, error, *in, linfit ;
int   numhyp, numin ;
double koutB[7], alpha,centerpt ;

 numin = P->numin ;  /* num theta  */
 numhyp = P->J[0].numhyp;
 c = (double *)calloc(numhyp*numin,sizeof(double));
 in = P->previn ;   /* theta  */
 h = (double *)calloc(numhyp,sizeof(double));
 for ( i = 0; i<P->numJout; i++ ) {
  P->Jout[i] = 0. ; 
  koutB[i] = 0.0 ;
 }

 for( a=0 ; a<P->numJin ; a++ )
 {
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   kout[a][b] = 0.0 ;
  }
 }

   mu = P->J[0].mu ;
   sig = P->J[0].sig ;
   hsum = 0. ;
   for ( j=0; j<numhyp; j++ )
   {
     diffmuin = 0. ;  
     for ( k=0; k<numin; k++ )
     {
      n = jk ;
      c[n] = (in[k] - mu[n])/sig[n] ;
      diffmuin += c[n]*c[n] ;
     }
     
     if( diffmuin < P->maxdiff )
      h[j] = exp( -diffmuin ) ;
     else
      h[j] = 0.0 ; 
     hsum += h[j] ;
   } 

   if(hsum > 0.0)
   {
    for ( j=0; j<numhyp; j++ ) {
     h[j] /= hsum ; }
   }
   else
    printf("Error\n");
  

/**********  Find J-1 entries  ******************************/

 for( a=0 ; a<P->numJin ; a++ )
 {
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   e = a*P->numJout + b ;
   w = P->J[e].w ;
   v = P->J[e].v ;
  

    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk]; 
      }
      kout[a][b] += h[j]*(v[j] + linfit) ;
    }
  }
 }

  /****  find the bias vector  **********/
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   w = P->B[b].w ;
   v = P->B[b].v ;
  
    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk]; 
      }
      koutB[b] += h[j]*(v[j] + linfit) ;
    }
  }

/*Now that we have Jinv entries and the bias vector, calc output vector**/


for( b=0 ; b<(P->numJout) ; b++ )
 {
  for( a=0 ; a<P->numJin ; a++ )
  {
   P->Jout[b] += P->Jin[a]*kout[a][b] ;
  }
  P->Jout[b] += koutB[b] ;
  /*  printf("%lf ",koutB[b]);*/
 }
/* printf("\n");*/

	    
/* apply gradient descent learning to rbfs that represent J-1 entries*/
 
 for( a=0 ; a<P->numJin ; a++ )
 {
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   e = a*P->numJout + b ;
   w = P->J[e].w ;
   v = P->J[e].v ;
   out = P->Jout[b] ;
   target = P->Jtarget[b] ;
   
    eta = P->etaP*P->Jin[a] ;
    meta = P->metaP*P->Jin[a] ;
    error = target - out ;
    for( j=0; j<numhyp; j++ )
    {
     if( h[j]>0.0 )
     {
      v[j] += eta*error*h[j] ;
      for( k=0; k<numin; k++ )
      {
       n = jk ;
       w[n] += eta*error*c[n]*h[j] ;


      }
     }
    }
  }

 }

/* apply gradient descent learning to rbfs that represent bias vector */
 
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   w = P->B[b].w ;
   v = P->B[b].v ;
   out = P->Jout[b] ;
   target = P->Jtarget[b] ;
   alpha = P->B[b].alpha ;
   
   centerpt = P->rangemin[b] + (P->range[b]/2.0);

   
    eta = P->etaB ;
    error = target - out ;
    
    for( j=0; j<numhyp; j++ )
    {
     if( h[j]>0.0 )
     {
 /*   v[j] += eta*(error - 
                  (alpha/(P->range[b]*P->range[b]))*(in[b]-centerpt))*h[j];
  */
        v[j] += eta*( error -  alpha*in[b])*h[j] ;

      for( k=0; k<numin; k++ )
      {
       
       n = jk ;
 /*    w[n] += eta*(error-
               (alpha/(P->range[b]*P->range[b]))*(in[b]-centerpt))*c[n]*h[j] ;
 */
        w[n] += eta*(error - alpha*in[b])*c[n]*h[j];


       }
     }
    }
}

free(c);
free(h);

}
/*********************  ApplyJInv() **************************************/
/*  This routine was cut from Pmap(). It performs the direction-to-direction
     mapping by updating the rbf structure P  

  precond:  1. P->previn must contain AMpar[]
            2. P->Jin must contain an array with an entry \delta x_i, 
               for each dimension i in the planning space

  postcond: P->Jout will contain an array with an entry \delta \theta_j  
               for each dimension j in articulator space


Warning:  Current version assumes the centers and sigs of the rbfs are in 
          the same places for all J & B entries - if adaptive rbfs are 
          used, must rewrite this code.
*/
void ApplyJInv( rbf_struct *P )
{
int  i, j, k, a, b, e, n ;
double diffmuin, hsum, *h ;
double kout[NUM_TARGETS][7] ;
double *c, *w, *v, *sig, *mu, error, *in, linfit ;
int   numhyp, numin ;
double koutB[7];

 numin = P->numin ;  /* num theta  */
 numhyp = P->J[0].numhyp;
 c = (double *)calloc(numhyp*numin,sizeof(double));
 in = P->previn ;   /* theta  */
 h = (double *)calloc(numhyp,sizeof(double));

 for ( i = 0; i<P->numJout; i++ ) {
  P->Jout[i] = 0. ; 
  koutB[i] = 0.0 ;
 }

 for( a=0 ; a<P->numJin ; a++ )
 {
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   kout[a][b] = 0.0 ;
  }
 }

   mu = P->J[0].mu ;
   sig = P->J[0].sig ;
   hsum = 0. ;
   for ( j=0; j<numhyp; j++ )
   {
     diffmuin = 0. ;  
     for ( k=0; k<numin; k++ )
     {
      n = jk ;
      c[n] = (in[k] - mu[n])/sig[n] ;
      diffmuin += c[n]*c[n] ;
     }
     
     if( diffmuin < P->maxdiff )
      h[j] = exp( -diffmuin ) ;
     else
      h[j] = 0.0 ; 
     hsum += h[j] ;
   } 

   if(hsum > 0.0)
   {
    for ( j=0; j<numhyp; j++ ) {
     h[j] /= hsum ; }
   }
   else
    printf("Error\n");
  

/**********  Find J-1 entries  ******************************/

 for( a=0 ; a<P->numJin ; a++ )
 {
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   e = a*P->numJout + b ;
   w = P->J[e].w ;
   v = P->J[e].v ;
  

    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk]; 
      }
      kout[a][b] += h[j]*(v[j] + linfit) ;
    }

  }
 }

  /****  find the bias vector  **********/
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   w = P->B[b].w ;
   v = P->B[b].v ;
  
    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk]; 
      }
      koutB[b] += h[j]*(v[j] + linfit) ;
    }
  }

/*Now that we have Jinv entries and the bias vector, calc output vector**/


for( b=0 ; b<(P->numJout) ; b++ )
 {
  for( a=0 ; a<P->numJin ; a++ )
  {
   P->Jout[b] += P->Jin[a]*kout[a][b] ;
  }
/*
  P->Jout[b] += koutB[b] ;
  if((b==6)||(b==2)){P->Jout[b] += 4*koutB[b];}else 
*/
  P->Jout[b] += 2.0*koutB[b] ;
  /* printf("%lf %lf\n",P->Jout[b],koutB[b]);*/
 }
free(c);free(h);
} 


/*************************** GetNearPos  *********************************

This routine moves a step in articulator space in the direction of 
P->direct and stores the new position in P->posn. If the new configuration 
is valid, the corresponding planning space posn is stored in P->M.target 
to act as the target for the forward map. If the new configuration is 
not valid, returns to the central configuration and picks a new random mvment
direction which it stores in P->direct.

PreConditions:

P->posn must be set to current artic posn. The vector P->direct must also 
be set to a normalized mvment step. To look for sharp changes in frmts need 
P->target set to current planning space posn upon entry (prevtarget violated)

StartTarget must contain the posn in planning space corresponding to the 
origin in artic space.

PostCond : P->posn will be set to new posn in artic space and P->M[].target to 
the corresponding posn in planning space

*/   

int GetNearPos ( rbf_struct *P, double *StartTarget )
{

int  k ;


  for ( k = 0; k<P->numin; k++ )
     {
    /* this is commented out so we can try just training around 0 
      P->posn[k] = 0.0;  
      P->prevtarget[k] = P->target[k];  to look for sharp frmt changes 
      */
      P->posn[k] += P->direct[k] ; 
      AMpar[k] = P->posn[k];
   }

  if(!check_valid_configuration()) {
/* U config
 P->posn[0] = 0.0;P->posn[1] = 1.5;P->posn[2] = -0.25; P->posn[3] = -0.1; 
 P->posn[4] = -1.2;P->posn[5] = 0.1; P->posn[6] = -0.4;
 */
      for (k=0;k<P->numin;k++) P->posn[k] = 0.0 ;
      for(k=0;k<P->numtarget;k++) P->target[k] = StartTarget[k] ;
      for (k=0;k<P->numin;k++) P->direct[k] = P->range[k]*(0.5-drand48())/20.0 ;
      printf("out\n");
      return(0);
   }

  get_formants() ;

  get_state(); /* this is just to see if state vars ok */ 

  for(k=0;k<P->numJin;k++)
    {
    P->target[k] = syn_targets[k] ;
 }
/*
    printf("%d %.1lf\n",k,P->target[k] - P->prevtarget[k]);
    if(fabs(P->target[k] - P->prevtarget[k]) > MAX_CHANGE[k]){
*/     
  

  for( k=0 ; k<P->numFMaps ; k++ )
    P->M[k].target = P->target[k]/*P->scale[k]*/ ;

 return(1);



}

 

/*************************** GetStep  *********************************

This routine finds a random valid configuration in artic space, which 
it stores in P->previn 

A random movement vector in artic space with magnitude=stepsize is found 
which causes movement to a new valid config P->in, this movement vector is 
stored in P->Jtarget as a target for the inverse map.  The before 
and after positions (previn,in) are mapped to frmt space (prevtarget, target) 
and from there mapped separately to a normalized state space where a diff 
vector is found and stored in P->Jin.
*/   

int GetStep ( rbf_struct *P )
{
 double amp[8];
 int num_formants;
 double total_frmt_change;
 double stepsize;

int j, k ;

 start:

  do{
     for ( k = 0; k<P->numin; k++ )
     {
    /* this is commented out so we can try just training around 0 
      P->previn[k] = 0.0;  
   */
      P->previn[k] = P->range[k]*drand48() + P->rangemin[k] ; 
      AMpar[k] = P->previn[k];
      P->direct[k] = drand48() - .5 ;
    }
 
  }while(!check_valid_configuration());

  NormVec( P->direct, P->numin ) ;

  get_formants() ;

  for(k=0;k<P->numJin;k++)
    P->prevtarget[k] = syn_targets[k] ;
  

  stepsize = P->stepsize;
  do{
      for ( k=0; k<P->numin; k++ ){
         P->in[k] = P->previn[k] + stepsize*P->direct[k] ;
         AMpar[k] = P->in[k];
      }
      stepsize = stepsize/2.0;
   }while((stepsize>P->stepsize/16.0)&&(!check_valid_configuration()));


   if(stepsize<=P->stepsize/16.0) goto start;
   stepsize = stepsize*2.0;

   get_formants() ;

   for(k=0;k<P->numJin;k++)
      P->target[k] = syn_targets[k] ;

  /* Now translate to state space and find difference vector: since 
    the translation to state space truncates the edges (discontinuous), 
    check if at the edge and if so, throw away the training pts & start 
    again. This should be rare because we axed all invalid configs. */

   get_state();
   
   
  for( j=0 ; ((j<P->numJin)&&(j<3)) ; j++){
     if((s[2*j+1]==0)||(s[2*j+1]==1)){
        printf("bad %d %d\n",j+1,s[2*j+1]);
        goto start;
     }
  }
  if(P->numJin > 3){
       if(s[7] == 0){
         printf("NO Voicing!!\n");
         goto start;
       }
     }
  
  for( j=0 ; j<P->numJin ; j++){
   P->Jin[j] = (s[2*j+1]-s[2*j+2]);  /*  Correct - begin indices at 0 */
  }

  for( j=0 ; j<P->numJin ; j++){
     syn_targets[j] = P->prevtarget[j];
   }
  get_state();


  for( j=0 ; ((j<P->numJin)&&(j<3)) ; j++){
     if((s[2*j+1]==0)||(s[2*j+1]==1)){
        printf("bad %d %d\n",j+1,s[2*j+1]);
        goto start;
     }
  }
  if(P->numJin > 3){
       if(s[7] == 0){
         printf("NO Voicing!!\n");
         goto start;
       }
     }
  
  for( j=0 ; j<P->numJin ; j++){
   P->Jin[j] -= (s[2*j+1]-s[2*j+2]);  /*  Correct - begin indices at 0 */

  }



  for( j=0 ; j<P->numJout ; j++)
     P->Jtarget[j] = stepsize*P->direct[j] ;

 return(1);



}
 
/*************************** NormVec *********************************/

void NormVec ( double *vec, int length )
{
 int k ;
 double norm ;
 double *start ;

 start = vec ;
 norm = 0. ;
 for ( k = 0; k<length; k++ )
 {
  norm += (*vec) * (*vec) ;
  vec++ ;
 }
 vec = start ;
 for ( k = 0; k<length; k++ )
 {
  *vec /= sqrt(norm) ;
  vec++ ;
 }
}


/**************************** SaveWeights ************************/

void SaveWeights (FILE *fp, rbf_struct *P )
{

 int a, i, j,numin,MAXHYP ;
 char marker_string[30];

  
  printf("Saving Weights: \n");

  sprintf(marker_string,"%%%%RbfWeightFile"); /* must double each % sign */
  fprintf(fp,"%s\n",marker_string);
  fprintf(fp,"\n");

  fprintf(fp,"%d\n",P->numFMaps);
  for(i=0;i<P->numFMaps;i++)
    fprintf(fp,"%d %lf %lf %lf\n",P->M[i].numhyp,P->M[i].target,P->M[i].out,P->M[i].error);
  fprintf(fp,"\n");

  fprintf(fp,"%d\n",P->numPMaps);
  for(i=0;i<P->numPMaps;i++)
     fprintf(fp,"%d ",P->J[i].numhyp);
  fprintf(fp,"\n");

  fprintf(fp,"%d\n",P->numBMaps);
  for(i=0;i<P->numBMaps;i++)
     fprintf(fp,"%lf %d\n",P->B[i].alpha,P->B[i].numhyp);

  fprintf(fp,"\n");
  fprintf(fp,"%d ",P->numEpochs);
  fprintf(fp,"%d ",P->numreach);
  fprintf(fp,"%d ",P->numstep);
  fprintf(fp,"%d ",P->MAXHYP);
  fprintf(fp,"%d ",P->numin);
  fprintf(fp,"%d ",P->numtarget);
  fprintf(fp,"%d ",P->numJin);
  fprintf(fp,"%d ",P->numJout);
  fprintf(fp,"%lf ",P->maxdiff);
  fprintf(fp,"%d ",P->epochs);
  fprintf(fp,"%lf ",P->stepsize);
  fprintf(fp,"%lf ",P->etaF);
  fprintf(fp,"%lf ",P->metaF);
  fprintf(fp,"%lf ",P->etaP);
  fprintf(fp,"%lf ",P->etaP_max);
  fprintf(fp,"%lf ",P->metaP);
  fprintf(fp,"%lf ",P->etaB);
  fprintf(fp,"%lf ",P->metaB);
  fprintf(fp,"%lf ",P->numsamp);

  fprintf(fp,"\n");

  numin = P->numin ;
  MAXHYP = P->MAXHYP;

  for(a=0 ; a<P->numFMaps ; a++ ) {
    for(i=0;i<numin;i++)
       fprintf(fp,"%d ",P->M[a].numseg[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->M[a].v[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP*numin;i++)
       fprintf(fp,"%lf ",P->M[a].w[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->M[a].mu[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->M[a].h[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->M[a].sig[i]);
    fprintf(fp,"\n");

 }


  for(a=0 ; a<P->numPMaps ; a++ ) {
    for(i=0;i<numin;i++)
       fprintf(fp,"%d ",P->J[a].numseg[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->J[a].v[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP*numin;i++)
       fprintf(fp,"%lf ",P->J[a].w[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->J[a].mu[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->J[a].h[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->J[a].sig[i]);
    fprintf(fp,"\n");

 }


  for(a=0 ; a<P->numBMaps ; a++ ) {
    for(i=0;i<numin;i++)
       fprintf(fp,"%d ",P->B[a].numseg[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->B[a].v[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP*numin;i++)
       fprintf(fp,"%lf ",P->B[a].w[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->B[a].mu[i]);
    fprintf(fp,"\n");

    for(i=0;i<MAXHYP;i++)
       fprintf(fp,"%lf ",P->B[a].h[i]);
    fprintf(fp,"\n");

    for(i=0;i<numin*MAXHYP;i++)
       fprintf(fp,"%lf ",P->B[a].sig[i]);
    fprintf(fp,"\n");

 }

  for(i=0;i<numin;i++)
   fprintf(fp,"%lf ",P->rangemin[i]);
  fprintf(fp,"\n");

  for(i=0;i<numin;i++)
   fprintf(fp,"%lf ",P->rangemax[i]);
  fprintf(fp,"\n");

  for(i=0;i<numin;i++)
   fprintf(fp,"%lf ",P->range[i]);
  fprintf(fp,"\n");

}



/********************************* ReadWeights ***********************************/


void ReadWeights (FILE *fp, rbf_struct *P )
{

 int a, i ;
 int numin , MAXHYP ;
 char instring[100];
  
  fscanf(fp,"%d",&(P->numFMaps));
  P->M = (FMap *)calloc(P->numFMaps, sizeof(FMap) ) ;
  for(i=0;i<P->numFMaps;i++)
     fscanf(fp,"%d %lf %lf %lf\n",&(P->M[i].numhyp),&(P->M[i].target),&(P->M[i].out),&(P->M[i].error));

  fscanf(fp,"%d",&(P->numPMaps));
  P->J = (PMap *)calloc(P->numPMaps, sizeof(PMap) ) ;
  for(i=0;i<P->numPMaps;i++)
     fscanf(fp,"%d",&(P->J[i].numhyp));



  fscanf(fp,"%d",&(P->numBMaps));
  P->B = (BMap *)calloc(P->numBMaps, sizeof(BMap) ) ;
  for(i=0;i<P->numBMaps;i++)
     fscanf(fp,"%lf %d",&(P->B[i].alpha),&(P->B[i].numhyp));


  fscanf(fp,"%d",&(P->numEpochs));
  fscanf(fp,"%d",&(P->numreach));
  fscanf(fp,"%d",&(P->numstep));
  fscanf(fp,"%d",&(P->MAXHYP));
  fscanf(fp,"%d",&(P->numin));
  fscanf(fp,"%d",&(P->numtarget));
  fscanf(fp,"%d",&(P->numJin));
  fscanf(fp,"%d",&(P->numJout));
  fscanf(fp,"%lf",&(P->maxdiff));
  fscanf(fp,"%d",&(P->epochs));
  fscanf(fp,"%lf",&(P->stepsize));
  fscanf(fp,"%lf",&(P->etaF));
  fscanf(fp,"%lf",&(P->metaF));
  fscanf(fp,"%lf",&(P->etaP));
  fscanf(fp,"%lf",&(P->etaP_max));
  fscanf(fp,"%lf",&(P->metaP));
  fscanf(fp,"%lf",&(P->etaB));
  fscanf(fp,"%lf",&(P->metaB));
  fscanf(fp,"%lf",&(P->numsamp));

  numin = P->numin ;
  MAXHYP = P->MAXHYP;

  for(a=0 ; a<P->numFMaps ; a++ ) {
    P->M[a].numseg = (int *)calloc(numin, sizeof(int) ) ;
    for(i=0;i<numin;i++)
       fscanf(fp,"%d",&(P->M[a].numseg[i]));

    P->M[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->M[a].v[i]));

    P->M[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
    for(i=0;i<MAXHYP*numin;i++)
       fscanf(fp,"%lf",&(P->M[a].w[i]));

    P->M[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->M[a].mu[i]));

    P->M[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->M[a].h[i]));

    P->M[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->M[a].sig[i]));
    
  }


  for(a=0 ; a<P->numPMaps ; a++ ) {
    P->J[a].numseg = (int *)calloc(numin, sizeof(int) ) ;
    for(i=0;i<numin;i++)
       fscanf(fp,"%d",&(P->J[a].numseg[i]));

    P->J[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->J[a].v[i]));

    P->J[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
    for(i=0;i<MAXHYP*numin;i++)
       fscanf(fp,"%lf",&(P->J[a].w[i]));

    P->J[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->J[a].mu[i]));

    P->J[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->J[a].h[i]));

    P->J[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->J[a].sig[i]));
    
  }


  for(a=0 ; a<P->numBMaps ; a++ ) {
    P->B[a].numseg = (int *)calloc(numin, sizeof(int) ) ;
    for(i=0;i<numin;i++)
       fscanf(fp,"%d",&(P->B[a].numseg[i]));

    P->B[a].v = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->B[a].v[i]));

    P->B[a].w = (double *)calloc(MAXHYP*numin, sizeof(double) ) ;
    for(i=0;i<MAXHYP*numin;i++)
       fscanf(fp,"%lf",&(P->B[a].w[i]));

    P->B[a].mu = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->B[a].mu[i]));

    P->B[a].h = (double *)calloc(MAXHYP, sizeof(double) ) ;
    for(i=0;i<MAXHYP;i++)
       fscanf(fp,"%lf",&(P->B[a].h[i]));

    P->B[a].sig = (double *)calloc(numin*MAXHYP, sizeof(double) ) ;
    for(i=0;i<numin*MAXHYP;i++)
       fscanf(fp,"%lf",&(P->B[a].sig[i]));
    
  }
  
  P->rangemin = (double *)calloc(numin,sizeof(double));
  for(i=0;i<numin;i++)
   fscanf(fp,"%lf",&(P->rangemin[i]));

  P->rangemax = (double *)calloc(numin,sizeof(double));
  for(i=0;i<numin;i++)
   fscanf(fp,"%lf",&(P->rangemax[i]));

  P->range = (double *)calloc(numin,sizeof(double));
  for(i=0;i<numin;i++)
   fscanf(fp,"%lf",&(P->range[i]));

  P->c = (double *)calloc(P->numin*P->MAXHYP, sizeof(double) ) ;
  P->linfit = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
  P->direct = (double *)calloc(P->numin, sizeof(double) ) ;
  P->in = (double *)calloc(P->numin, sizeof(double) ) ;
  P->previn = (double *)calloc(P->numin, sizeof(double) ) ;
  P->posn = (double *)calloc(P->numin, sizeof(double) ) ;
  P->target = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->prevtarget = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->avgJin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->Jout = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jtarget = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jerror = (double *)calloc(P->numJout, sizeof(double) ) ;

}




/******************  RbfInvMap**************************************
    Here is the RBF routine which maps changes in planning
    space (x) to changes in articulators (m). 

  precond:
    1.  An up-to date AMpar array must be passed (necessary because 
        the inverse Jacobian depends on artic config)
    2.  The RBF structure P must have been created, initialized and trained 
        before this function is called.  This is currently done by 
        function calls made from the talker.c file 
    3.  The parameter x[] (from the DIVA code) starts at index 1, and has
        agonist-antagonist entries, one of which is 0.

  postcond:
    The array m will contain desired changes in artic config, with indices 
    starting at zero and agonist-antagonist entries as in DIVA.  The first 
    two parameters are not changed by RbfInvMap(), they are only used 
    to update m[].

 
*/


void RbfInvMap(x,AMpar,m,P)
double *x, *AMpar, *m;
rbf_struct *P;
{
 int i; 
  
  
  if((P->numin == 7)&&(P->numJin==(NUM_STATE_VARS/2))){
    for (i=0;i<7;i++){
        P->previn[i] = AMpar[i];
    }
    for(i=0;i<NUM_TARGETS;i++){
        P->Jin[i] = x[2*i+1]- x[2*i+2];      /* Correct indices of x */
     }
 }
  else{
    printf("ERROR: P inverse map has incompatible dimensionality\n");
    exit(0);
 }
  
 ApplyJInv(P);

 if(P->numJout == 7){
   for(i=0;i<7;i++){
      if(P->Jout[i]>0){
          m[2*i+1] = P->Jout[i];
          m[2*i+2] = 0.0;
       }
      else{
          m[2*i+1] = 0.0;
          m[2*i+2] = -P->Jout[i];
       }
   }
}
  else{
    printf("ERROR: P->Jout has incompatible dimensionality\n");
    exit(0);
 }

}


/******************  RbfForMap**************************************
    Here is the RBF routine which maps articulator position (AMpar)
    to position in planning space.

  precond:
    1.  An up-to date AMpar array must be passed (necessary because 
        the Jacobian depends on artic config)
    2.  The RBF structure P must have been created, initialized and trained 
        before this function is called.  This is currently done by 
        function calls made from the talker.c file 

  postcond:
    The array plan_pos will contain corresponding position in planning space.  
    The first parameter (AMpar) is not changed by RbfForMap(), it's only 
    used to update plan_pos[].


*/


void RbfForMap(AMpar,plan_pos,P)
double *plan_pos, *AMpar;
rbf_struct *P;
{
 int i; 
   
  if((P->numin == 7)&&(P->numFMaps==(NUM_STATE_VARS/2))){
    for (i=0;i<7;i++){
        P->posn[i] = AMpar[i];
    }
  }
  else{
    printf("ERROR: P forward map has incompatible dimensionality\n");
    exit(0);
 }
  
 ApplyForMap(P);

 for(i=0;i<NUM_TARGETS;i++){
      plan_pos[i] = P->M[i].out;
   }
}


/********************  ApplyForMap  *************************************
Cut out of Fmap()


Warning:  Current version assumes the centers and sigs of the rbfs are in 
          the same places for all M entries - if adaptive rbfs are 
          used, must rewrite this code, esp watch use of P->c.

precond:  P->posn must contain articulator config

postcond:  P->M[a].out will contain the ath planning space variable
*/



void ApplyForMap( rbf_struct *P )
{
 int  i, j, k, a, b, n ;
 double diffmuin, hsum, *h ;
 double eta, meta, dmu, out, target, error ;
 double *c, *w, *v, *sig, *mu, *in, linfit ;
 int   numhyp, numin ;

  numin = P->numin ;
  numhyp = P->M[0].numhyp;
  c = (double *)calloc(numin*numhyp,sizeof(double));
  h = (double *)calloc(numhyp,sizeof(double));
  in = P->posn ;
  eta = P->etaF ;
  meta = P->metaF ;
   

   mu = P->M[0].mu ;
   sig = P->M[0].sig ;
   hsum = 0. ;
   for ( j=0; j<numhyp; j++ )
   {
     diffmuin = 0. ;  
     for ( k=0; k<numin; k++ )
     {
      n = jk ;
      c[n] = (in[k] - mu[n])/sig[n] ;
      diffmuin += c[n]*c[n] ;
     }
     
     if( diffmuin < P->maxdiff ) 
      h[j] = exp( -diffmuin ) ;
     else
      h[j] = 0.0 ; 
     hsum += h[j] ;     
   }     

   if(hsum > 0.0)
   {
    for ( j=0; j<numhyp; j++ ) {
     h[j] /= hsum ; }
   }
   else
    printf("Error\n") ;

  for(a = 0; a<P->numFMaps; a++)
  {
   w = P->M[a].w ;
   v = P->M[a].v ;
   target = P->M[a].target ;
   

   out = 0. ;
  
 
    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk] ;
      }
      out += h[j]*(v[j] + linfit) ;
    }
     
   P->M[a].out = out/*P->scale[a]*/ ;  
   }  /* end for a */
free(c);free(h);
}



void FreeRbf( rbf_struct *P )
{
 int a;   
  
  for(a=0 ; a<P->numFMaps ; a++ ) {
     free(P->M[a].numseg);
     free(P->M[a].v);
     free(P->M[a].w);
     free(P->M[a].mu);
     free(P->M[a].h);
     free(P->M[a].sig);
  }  
  for(a=0 ; a<P->numPMaps ; a++ ) {
     free(P->J[a].numseg);  
     free(P->J[a].v);
     free(P->J[a].w);
     free(P->J[a].mu);
     free(P->J[a].h);
     free(P->J[a].sig);
  }
  for(a=0 ; a<P->numBMaps ; a++ ) {
     free(P->B[a].numseg);  
     free(P->B[a].v);
     free(P->B[a].w);
     free(P->B[a].mu);
     free(P->B[a].h);
     free(P->B[a].sig);
  }
  free(P->c);free(P->linfit);free(P->direct);free(P->in);
  free(P->previn);free(P->posn);free(P->target);free(P->prevtarget);
  free(P->Jin);free(P->avgJin);free(P->Jout);
  free(P->Jtarget);free(P->Jerror);

  free(P->M);
  free(P->J);
  free(P->B);
 

}





/****************************************************/

void OldSaveWeights (FILE *fp, rbf_struct *P )
{

 int a, i, j,numin ;
 char marker_string[30];

  fwrite(P, sizeof(rbf_struct), 1, fp) ;

  fwrite(P->M, sizeof(FMap), P->numFMaps, fp) ;
  fwrite(P->J, sizeof(PMap), P->numPMaps, fp) ;
  fwrite(P->B, sizeof(BMap), P->numBMaps, fp) ;

  for(a=0 ; a<P->numFMaps ; a++ ) 
    fwrite(P->M[a].numseg, sizeof(int), P->numin, fp) ;
  
  for(a=0 ; a<P->numPMaps ; a++ )
    fwrite(P->J[a].numseg, sizeof(int), P->numin, fp) ;

  for(a=0 ; a<P->numBMaps ; a++ )

    fwrite(P->B[a].numseg, sizeof(int), P->numin, fp) ;
  
  fwrite(P->rangemin,sizeof(double),P->numin,fp);  
  fwrite(P->rangemax,sizeof(double),P->numin,fp);  
  fwrite(P->range,sizeof(double),P->numin,fp);  



  for(a=0; a<P->numFMaps; a++)
  {
    numin = P->numin ;

    fwrite(P->M[a].v, sizeof(double), P->MAXHYP, fp) ;
    fwrite(P->M[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;	  
    fwrite(P->M[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
    fwrite(P->M[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;
  }
	 
  for(a=0; a<P->numPMaps; a++)
  {
   numin = P->numin ;

   fwrite(P->J[a].v, sizeof(double), P->MAXHYP, fp) ;
   fwrite(P->J[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;	  
   fwrite(P->J[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
   fwrite(P->J[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;

  }  

  for(a=0; a<P->numBMaps; a++)
  {
   numin = P->numin ;

   fwrite(P->B[a].v, sizeof(double), P->MAXHYP, fp) ;
   fwrite(P->B[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;	  
   fwrite(P->B[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
   fwrite(P->B[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;

  }  
}
void NewInvMap(rbf_struct *P)
{
int i,j,k,numin;
 
numin = P->numin;
P->etaB = 0.05;
for(i=0;i<7;i++){
  P->B[i].alpha = 0.01;
}
for(i=0;i<P->numPMaps;i++){

  for(j=0;j<P->MAXHYP;j++){
   P->J[i].v[j] = 0.0;
   for(k=0;k<P->numin;k++)
     P->J[i].w[jk] = 0.0;
  }
}
for(i=0;i<P->numBMaps;i++){
  for(j=0;j<P->MAXHYP;j++){
   P->B[i].v[j] = 0.0;
   for(k=0;k<P->numin;k++)
     P->B[i].w[jk] = 0.0;
  }
}  
}
void OldReadWeights (FILE *fp, rbf_struct *P )
{

 int a, i ;
 int numin ;
 char instring[100];
  
  fread(P, sizeof(rbf_struct), 1, fp) ;


  P->M = (FMap *)calloc(P->numFMaps, sizeof(FMap) ) ;
  P->J = (PMap *)calloc(P->numPMaps, sizeof(PMap) ) ;
  P->B = (BMap *)calloc(P->numBMaps, sizeof(BMap) ) ;

  fread(P->M, sizeof(FMap), P->numFMaps, fp) ;
  fread(P->J, sizeof(PMap), P->numPMaps, fp) ;
  fread(P->B, sizeof(BMap), P->numBMaps, fp) ;

  for(a=0 ; a<P->numFMaps ; a++ ) {
    P->M[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ;
    fread(P->M[a].numseg, sizeof(int), P->numin, fp) ;
  }
  
  for(a=0 ; a<P->numPMaps ; a++ ) {
    P->J[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ;  
    fread(P->J[a].numseg, sizeof(int), P->numin, fp) ;   
  }
  
  for(a=0 ; a<P->numBMaps ; a++ ) {
    P->B[a].numseg = (int *)calloc(P->numin, sizeof(int) ) ;  
    fread(P->B[a].numseg, sizeof(int), P->numin, fp) ;   
  }

  P->rangemin = (double *)calloc(P->numin,sizeof(double));
  fread(P->rangemin,sizeof(double),P->numin,fp);

  P->rangemax = (double *)calloc(P->numin,sizeof(double));
  fread(P->rangemax,sizeof(double),P->numin,fp);

  P->range = (double *)calloc(P->numin,sizeof(double));
  fread(P->range,sizeof(double),P->numin,fp);

  P->c = (double *)calloc(P->numin*P->MAXHYP, sizeof(double) ) ;
  P->linfit = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
  P->direct = (double *)calloc(P->numin, sizeof(double) ) ;
  P->in = (double *)calloc(P->numin, sizeof(double) ) ;
  P->previn = (double *)calloc(P->numin, sizeof(double) ) ;
  P->posn = (double *)calloc(P->numin, sizeof(double) ) ;
  P->target = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->prevtarget = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->avgJin = (double *)calloc(P->numJin, sizeof(double) ) ;
  P->Jout = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jtarget = (double *)calloc(P->numJout, sizeof(double) ) ;
  P->Jerror = (double *)calloc(P->numJout, sizeof(double) ) ;


 for(a=0; a<P->numFMaps; a++)
  {
    numin = P->numin ;
    
    P->M[a].v = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->M[a].w = (double *)calloc(P->MAXHYP*numin, sizeof(double) ) ;
    P->M[a].mu = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;
    P->M[a].h = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->M[a].sig = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;

    fread(P->M[a].v, sizeof(double), P->MAXHYP, fp) ;
    fread(P->M[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;
    fread(P->M[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
    fread(P->M[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;

  }
	 
  for(a=0; a<P->numPMaps; a++)
  {
   numin = P->numin ;

    P->J[a].v = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->J[a].w = (double *)calloc(P->MAXHYP*numin, sizeof(double) ) ;
    P->J[a].mu = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;
    P->J[a].h = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->J[a].sig = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;

    fread(P->J[a].v, sizeof(double), P->MAXHYP, fp) ;
    fread(P->J[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;
    fread(P->J[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
    fread(P->J[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;

  }  
  for(a=0; a<P->numBMaps; a++)
  {
   numin = P->numin ;

    P->B[a].v = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->B[a].w = (double *)calloc(P->MAXHYP*numin, sizeof(double) ) ;
    P->B[a].mu = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;
    P->B[a].h = (double *)calloc(P->MAXHYP, sizeof(double) ) ;
    P->B[a].sig = (double *)calloc(numin*P->MAXHYP, sizeof(double) ) ;

    fread(P->B[a].v, sizeof(double), P->MAXHYP, fp) ;
    fread(P->B[a].w, sizeof(double), P->MAXHYP*P->numin, fp) ;
    fread(P->B[a].mu, sizeof(double), P->numin*P->MAXHYP, fp) ;
    fread(P->B[a].sig, sizeof(double), P->numin*P->MAXHYP, fp) ;

  }  

}


void PrintBias( rbf_struct *P, double in[7] )
{
int  i, j, k, a, b, e, n ;
double diffmuin, hsum, *h ;
double kout[NUM_TARGETS][7] ;
double *c, *w, *v, *sig, *mu, error,  linfit ;
int   numhyp, numin ;
double koutB[7];

 numin = P->numin ;  /* num theta  */
 numhyp = P->J[0].numhyp;
 c = (double *)calloc(numhyp*numin,sizeof(double));
  h = (double *)calloc(numhyp,sizeof(double));

 for ( i = 0; i<P->numJout; i++ ) {
  koutB[i] = 0.0 ;
 }

   mu = P->J[0].mu ;
   sig = P->J[0].sig ;
   hsum = 0. ;
   for ( j=0; j<numhyp; j++ )
   {
     diffmuin = 0. ;  
     for ( k=0; k<numin; k++ )
     {
      n = jk ;
      c[n] = (in[k] - mu[n])/sig[n] ;
      diffmuin += c[n]*c[n] ;
     }
     
     if( diffmuin < P->maxdiff )
      h[j] = exp( -diffmuin ) ;
     else
      h[j] = 0.0 ; 
     hsum += h[j] ;
   } 

   if(hsum > 0.0)
   {
    for ( j=0; j<numhyp; j++ ) {
     h[j] /= hsum ; }
   }
   else
    printf("Error\n");
  

  /****  find the bias vector  **********/
  for( b=0 ; b<(P->numJout) ; b++ )
  {
   w = P->B[b].w ;
   v = P->B[b].v ;
  
    for ( j = 0; j<numhyp; j++ )
    {
      linfit = 0. ;
      for ( k = 0; k<numin; k++ )
      {
        linfit += w[jk]*c[jk]; 
      }
      koutB[b] += h[j]*(v[j] + linfit) ;

    }
   printf("%lf ",koutB[b]);
  }
  printf("\n");

free(c);free(h);
}
