#include <stdio.h>
#include <ctype.h>
#include "../main_src/diva.h"
#include "sensyn.h" /* Defines all variables in capital letters */
#include "../main_src/diva_dir.h"
#include "def_sensyn.h"

extern int defval[NPAR];	/* Default values for all parameters */
extern char cv[NPAR];		/* Constant/variable switch for each param */
extern int pdata[MAX_VAR_PARS][MAX_FRAMES];  /* Parameter data [np][nf] */
extern char dname[40];		/* .doc configuration file name */
extern int nframes;		/* # frames (Dur in ms divided by framedur */
extern int npars;		/* Number of parameters in config definition */

int	utterance_length = 4;	/* in mS; frame_length should equal */
                                /* utterance_length, and 128 must be */
                                /* evenly divisible by this number */
int	frame_length = 4;       /* Right now this should be 4 ms, to insure */
                                /* 32 sensyn frames per sound buffer */

init_sensyn( void )
{
	initconfig();		/* Read defaults from .con file */
	initpars();		/* Put defaults in time functions */

	cv[def_F0] = VARRIED;
	cv[def_F1] = VARRIED;
	cv[def_F2] = VARRIED;
	cv[def_F3] = VARRIED;
	cv[def_F4] = VARRIED;
	cv[def_AV] = VARRIED;
	cv[def_AF] = VARRIED;
#ifdef COMMENT
	cv[def_AH] = VARRIED;
#endif COMMENT
}


/*
 * The targets are the formants, then noise parameters
 */
sensyn( int *target )
{
	static	int	oldtarget[NUM_TARGETS];
	static  int     oldF0,oldAF,oldAV;
	static	int	first_time_flag=0;

	int	i;
	double	m[NUM_TARGETS];		/* slope for interpolating target */
	double  mF0,mAF,mAV;
	double	temp;


        /*don't blow Frank's speakers - reduce voicing amp*/
        if(NUM_TARGETS > 3) target[3] = target[3]/10.0;

	if (first_time_flag == 0) {
		oldF0 = (int)(10.0*F0);     /* Sensyn F0 is in 1/10 hz */
		for(i=0; i<NUM_TARGETS; i++)
			oldtarget[i] = target[i];
		first_time_flag++;
                oldAF = AF;
                if(NUM_TARGETS < 4) oldAV = AV;
	}

#ifdef COMMENT
for (i=0; i<npars; i++)
printf("defval[%d] = %d\n", i, defval[i]);

	defval[def_A1V] = 50;		/* amplitude */
	defval[def_A2V] = 45;		/* amplitude */
	defval[def_A3V] = 40;		/* amplitude */
	defval[def_A4V] = 35;		/* amplitude */
#endif COMMENT

	nframes = utterance_length/frame_length;

	for(i=0; i<NUM_TARGETS; i++)
	    m[i] = ((double)target[i]-(double)oldtarget[i])/(double)nframes;
	mF0 = ((F0 * 10.0) - (double)oldF0)/(double)nframes;
        mAF = ((double)AF - (double)oldAF)/(double)nframes;
        if(NUM_TARGETS <4)
           mAV = ((double)AV - (double)oldAV)/(double)nframes;

	for(i=0; i<nframes; i++) {

                /* amplitude of frication noise */
                temp = (double)oldAF + mAF*i;
                pdata[def_AF][i] = temp;


		/* F0 */
		temp = (double)oldF0 + mF0*i;
		pdata[def_F0][i] = temp;

		/* F1 */
		temp = (double)oldtarget[0] + m[0]*i;
		pdata[def_F1][i] = temp;

		/* F2 */
		temp = (double)oldtarget[1] + m[1]*i;
		pdata[def_F2][i] = temp;

		/* F3 and F4 (F4 is not set in the target) */
		temp = (double)oldtarget[2] + m[2]*i;
		pdata[def_F3][i] = temp;
		temp = (double)pdata[def_F3][i] + 1000.0;
		pdata[def_F4][i] = temp;

              if(NUM_TARGETS > 3){
		/* amplitude of voicing */
		temp = (double)oldtarget[3] + m[3]*i;
		pdata[def_AV][i] = temp;
	      }
              else{
               /* amplitude of voicing */
                temp = (double)oldAV + mAV*i;
                pdata[def_AV][i] = temp;
	      }

#ifdef COMMENT
		/* amplitude of frication noise */
		temp = (double)oldtarget[4] + m[4]*i;
		pdata[def_AF][i] = temp;

		/* amplitude of aspiration noise */
		temp = (double)oldtarget[5] + m[5]*i;
		pdata[def_AH][i] = temp;
#endif COMMENT
	}

	synthesize();

	for(i=0; i<NUM_TARGETS; i++)
		oldtarget[i] = target[i];
	oldF0 = (int)(10.0 * F0);
        oldAF = AF;
        if(NUM_TARGETS < 4) oldAV = AV;
}

