/*
 *      SenSyn - Version 1.0
 *
 *      Copyright (c) 1991, 1992 by Sensimetrics Corporation
 *      All Rights Reserved.
 *
 */

/*
 *  reson.c -
 *    This code implements a second order resonator, which can be used
 *    to realize a pole pair or zero pair.
 *
 *  coded by Andrew Wilson Howitt
 *
 *  Modification History:
 *    26 Sep 1991	E.P.Carlson, added AdvanceAntiResonator and InterPolePair
 */

#include <math.h>
#include "reson.h"

#define PI 3.1415926

void ClearResonator(Resonator *theResonator)
{
  theResonator->State.Z1 = 0.;
  theResonator->State.Z2 = 0.;
}

Float AdvanceResonator(Resonator *theResonator, Float input)
{
  register Float temp1,temp2;

  temp1 = theResonator->Coef.C * theResonator->State.Z2;
  theResonator->State.Z2 = theResonator->State.Z1;

  temp2 = theResonator->Coef.B * theResonator->State.Z1;
  temp1 += temp2;

  temp2 = theResonator->Coef.A * input;
  theResonator->State.Z1 = temp2 + temp1;

  return theResonator->State.Z1;
}

Float AdvanceAntiResonator(Resonator *theResonator, Float input)
{
  register Float temp1,temp2;

  temp1 = theResonator->Coef.C * theResonator->State.Z2;
  theResonator->State.Z2 = theResonator->State.Z1;

  temp2 = theResonator->Coef.B * theResonator->State.Z1;
  temp1 += temp2;

  temp2 = theResonator->Coef.A * input;
  theResonator->State.Z1 = input;

  return temp2 + temp1;
}

void SetPolePair(Resonator *theResonator, int CF, int BW, int SF)
{
  register double magnitude,angle,PiT;

  PiT = PI / (double)SF;
  magnitude = exp(-PiT * (double)BW);
  angle = 2. * PiT * (double)CF;

  theResonator->Coef.C = (Float)(-magnitude * magnitude);
  theResonator->Coef.B = (Float)(magnitude * cos(angle) * 2.);
  theResonator->Coef.A = 1. - theResonator->Coef.B - theResonator->Coef.C;
}

void InterPolePair(Resonator *theResonator, int CF, int BW, int SF)
{
  register double coefA, temp;

  coefA = theResonator->Coef.A;
  SetPolePair(theResonator, CF, BW, SF);
  if ((coefA != theResonator->Coef.A) && coefA) {
    temp = sqrt(theResonator->Coef.A / coefA);
    theResonator->State.Z1 *= temp;
    theResonator->State.Z2 *= temp;
  }
}

void SetZeroPair(Resonator *theResonator, int CF, int BW, int SF)
{
  SetPolePair(theResonator,CF,BW,SF);
  theResonator->Coef.A = 1. / theResonator->Coef.A;
  theResonator->Coef.B *= -theResonator->Coef.A;
  theResonator->Coef.C *= -theResonator->Coef.A;
}
