/*
 *      SenSyn - Version 1.1
 *
 *      Copyright (c) 1991, 1992 by Sensimetrics Corporation
 *      All Rights Reserved.
 *
 */

/*
 *  sample.c - synthesize one sample
 *
 *  coded by Eric P. Carlson
 *
 *  Modification History:
 *    14 Jan 1993       sign-extend modulo in noise calculation
 *     8 Jan 1993	corrected modulo arithmetic in noise calculation
 *    10 Mar 1992	removed aspiration modulation when F0 = 0
 *     5 Nov 1991	removed from spyn.c
 *     7 Nov 1991	moved voicing source to voice.c
 */

#include <math.h>		/* needed for power function */
#include "reson.h"		/* resonator support */
#include "synth.h"		/* synthesizer support */


/* ---------------------------------------------------------------------- */

Float next_sample(Coefficients *coefs)
{
  Float noise, first_diff, output, special;
  register Float casc;
  

  out[O_VOICING] = next_voice_sample();


/* ---------------------------------------------------------------------- */
/* aspiration/frication noise source */
  noise = (int)(short)(synth.random = ((synth.random * 20077L + 12345L)
				       % 65536L)) / 65536.;
  if (pars[P_F0] && !synth.glottis_open && synth.AV)
    noise /= 2;
  out[O_ASPIRATION] = coefs->asp_amp * noise - synth.asp_state;
  synth.asp_state = coefs->asp_amp * noise;
  out[O_GLOTTAL] = 12. * out[O_VOICING] + 5. * out[O_ASPIRATION];
  out[O_FRICATION] = -(coefs->fric_amp * noise);


/* ---------------------------------------------------------------------- */
/* cascade synthesis */

  if (!synth.parallel_only_flag) {
    casc = out[O_NASAL_POLE_CASC] = AdvanceResonator(&nasal_pole_cascade,
      out[O_NASAL_ZERO_CASC] = AdvanceAntiResonator(&nasal_zero_cascade,
        out[O_TRACHEAL_CASC] = AdvanceResonator(&trach_pole_cascade,
			         AdvanceAntiResonator(&trach_zero_cascade,
				   out[O_GLOTTAL] / 4.))));
    switch (spkrdef[S_NF]) {
      case 8:    casc = AdvanceResonator(&formant_8_cascade, casc);
      case 7:    casc = AdvanceResonator(&formant_7_cascade, casc);
      case 6:    casc = AdvanceResonator(&formant_6_cascade, casc);
      case 5:    casc = out[O_FORMANT_5_CASC] =
		   AdvanceResonator(&formant_5_cascade, casc);
      case 4:    casc = out[O_FORMANT_4_CASC] =
		   AdvanceResonator(&formant_4_cascade, casc);
      case 3:    casc = out[O_FORMANT_3_CASC] =
		   AdvanceResonator(&formant_3_cascade, casc);
      case 2:    casc = out[O_FORMANT_2_CASC] =
		   AdvanceResonator(&formant_2_cascade, casc);
      case 1:
      default:   out[O_FORMANT_1_CASC] =
		   AdvanceResonator(&formant_1_cascade, casc);
    }

    /* if using cascade, clear state of special parallel branch */
    synth.glottal_state = first_diff = special = 0;
  }


/* ---------------------------------------------------------------------- */
/* special parallel branch */

  else {

    first_diff = out[O_GLOTTAL] - synth.glottal_state;
    synth.glottal_state = out[O_GLOTTAL];

    special = (out[O_NASAL_PARA] = AdvanceResonator(&nasal_pole_special,
				    out[O_GLOTTAL] * coefs->npv_amp))
            + (out[O_FORMANT_1_PARA] = AdvanceResonator(&formant_1_special,
				    out[O_GLOTTAL] * coefs->f1v_amp))
	    - AdvanceResonator(&formant_2_special,
		    first_diff * coefs->f2v_amp)
	    + AdvanceResonator(&formant_3_special,
		    first_diff * coefs->f3v_amp)
	    - AdvanceResonator(&formant_4_special,
		    first_diff * coefs->f4v_amp)
	    + AdvanceResonator(&trach_pole_special,
		    first_diff * coefs->tpv_amp);
  }


/* ---------------------------------------------------------------------- */
/* parallel synthesis */

  out[O_FORMANT_2_PARA] = AdvanceResonator(&formant_2_parallel,
	                     out[O_FRICATION] * coefs->f2p_amp);
  out[O_FORMANT_3_PARA] = AdvanceResonator(&formant_3_parallel,
	                     out[O_FRICATION] * coefs->f3p_amp);
  out[O_FORMANT_4_PARA] = AdvanceResonator(&formant_4_parallel,
	                     out[O_FRICATION] * coefs->f4p_amp);
  out[O_FORMANT_5_PARA] = AdvanceResonator(&formant_5_parallel,
			     out[O_FRICATION] * coefs->f5p_amp);
  out[O_FORMANT_6_PARA] = AdvanceResonator(&formant_6_parallel,
			     out[O_FRICATION] * coefs->f6p_amp);
  out[O_BYPASS_PARA] = out[O_FRICATION] * coefs->bypass_amp;


/* ---------------------------------------------------------------------- */
/* normal output */

  out[O_NORMAL] = out[O_FORMANT_1_CASC] +
		  out[O_FORMANT_2_PARA] - out[O_FORMANT_3_PARA] +
		  out[O_FORMANT_4_PARA] - out[O_FORMANT_5_PARA] +
		  out[O_FORMANT_6_PARA] - out[O_BYPASS_PARA] + special;

  switch (spkrdef[S_OS]) {
	case 1:  
	case 2:
	case 3:	 output = out[spkrdef[S_OS]] + 0.99 * synth.integrator;
		 synth.integrator = output;
		 break;

	default:  output = out[spkrdef[S_OS]];
	          break;
  }
  return output;
}
